@@ -1,53 +1,38 @@
 from __future__ import annotations
 
-import json
-import re
 import sys
 from contextlib import contextmanager
-from enum import Enum
-from importlib import import_module, util
+from importlib import util
 from logging import getLogger
 from pathlib import Path
 from textwrap import dedent
-from typing import (
-    TYPE_CHECKING,
-    Callable,
-    Iterator,
-    NamedTuple,
-    Optional,
-    Sequence,
-    Union,
-)
+from typing import TYPE_CHECKING, Iterator, NamedTuple, Optional, Sequence, Union
 
-import pytomlpp as toml
-import yaml
-from pydantic import BaseModel, Extra, Field, ValidationError, root_validator, validator
+from pydantic import Extra, Field, ValidationError, root_validator, validator
+from pydantic.error_wrappers import ErrorWrapper
+from pydantic.main import BaseModel, ModelMetaclass
 
+from ..types import PythonName
+from . import _validators
+from ._bases import ImportExportModel
 from .contributions import ContributionPoints
 from .package_metadata import PackageMetadata
 from .utils import Version
 
 try:
-    from importlib.metadata import Distribution, distributions
+    from importlib import metadata
 except ImportError:
-    from importlib_metadata import Distribution, distributions  # type: ignore
+    import importlib_metadata as metadata  # type: ignore
 
 if TYPE_CHECKING:
     from importlib.metadata import EntryPoint
 
-spdx_ids = (Path(__file__).parent / "spdx.txt").read_text().splitlines()
-SPDX = Enum("SPDX", {i.replace("-", "_"): i for i in spdx_ids})  # type: ignore
 
 logger = getLogger(__name__)
 
-ENTRY_POINT = "napari.manifest"
 
-# The first release of npe2 defines the first engine version: 0.1.0.
-# As the contract around plugins evolve the ENGINE_NUMBER should be
-# increased follow SemVer rules. Note that sometimes the version number
-# will change even though no npe2 code changes.
-ENGINE_VERSION = "0.1.0"
-_display_name_pattern = re.compile(r"^[^\W_][\w -~]{1,38}[^\W_]$")
+SCHEMA_VERSION = "0.1.0"
+ENTRY_POINT = "napari.manifest"
 
 
 class DiscoverResults(NamedTuple):
@@ -56,7 +41,7 @@ class DiscoverResults(NamedTuple):
     error: Optional[Exception]
 
 
-class PluginManifest(BaseModel):
+class PluginManifest(ImportExportModel):
 
     # VS Code uses <publisher>.<name> as a unique ID for the extension
     # should this just be the package name ... not the module name? (yes)
@@ -65,8 +50,11 @@ class PluginManifest(BaseModel):
 
     name: str = Field(
         ...,
-        description="The name of the plugin. Should correspond to the python "
-        "package name for this plugin.",
+        description="The name of the plugin. Though this field is mandatory, it *must*"
+        " match the package `name` as defined in the python package metadata.",
+    )
+    _validate_name = validator("name", pre=True, allow_reuse=True)(
+        _validators.package_name
     )
 
     display_name: str = Field(
@@ -76,24 +64,27 @@ class PluginManifest(BaseModel):
         # and must not begin or end with an underscore, white space, or
         # non-word character.
     )
+    _validate_display_name = validator("display_name", allow_reuse=True)(
+        _validators.display_name
+    )
 
     # Plugins rely on certain guarantees to interoperate propertly with the
     # plugin engine. These include the manifest specification, conventions
     # around python packaging, command api's, etc. Together these form a
-    # "contract". The version of this contract is the "engine version."
+    # "contract". The version of this contract is the "schema version."
     #
-    # The first release of npe2 defines the first engine version.
-    # As the contract around plugins evolve the ENGINE_NUMBER should be
+    # The first release of npe2 defines the first schema version.
+    # As the contract around plugins evolve the SCHEMA_VERSION should be
     # increased follow SemVer rules. Note that sometimes the version number
     # will change even though no npe2 code changes.
     #
-    # The `engine` field declares the version of the contract that this plugin
-    # targets. Since there is no other version at the moment, it defaults to
-    # the current npe2 engine version.
-    engine: str = Field(
-        ENGINE_VERSION,
-        description="A SemVer compatible version string matching the versions "
-        "of the plugin engine that the extension is compatible with.",
+    # The `schema_version` field declares the version of the contract that this
+    # plugin targets.
+    schema_version: str = Field(
+        SCHEMA_VERSION,
+        description="A SemVer compatible version string matching the napari plugin "
+        "schema version that the plugin is compatible with.",
+        always_export=True,
     )
 
     # TODO:
@@ -101,29 +92,48 @@ class PluginManifest(BaseModel):
     # the actual mechanism/consumption of plugin information) independently
     # of napari itself
 
-    # TODO: refactor entry_point to binding points for activate,deactivate
-    # TODO: Point to activate function
-    # TODO: Point to deactivate function
-
-    # The module that has the activate() function
-    entry_point: Optional[str] = Field(
+    on_activate: Optional[PythonName] = Field(
         default=None,
-        description="The extension entry point. This should be a fully "
-        "qualified module string. e.g. `foo.bar.baz` for a module containing "
-        "the plugin's activate() function.",
+        description="Fully qualified python path to a function that will be called "
+        "upon plugin activation (e.g. `'my_plugin.some_module:activate'`). The "
+        "activate function can be used to connect command ids to python callables, or"
+        " perform other side-effects. A plugin will be 'activated' when one of its "
+        "contributions is requested by the user (such as a widget, or reader).",
+    )
+    _validate_activate_func = validator("on_activate", allow_reuse=True)(
+        _validators.python_name
+    )
+    on_deactivate: Optional[PythonName] = Field(
+        default=None,
+        description="Fully qualified python path to a function that will be called "
+        "when a user deactivates a plugin (e.g. `'my_plugin.some_module:deactivate'`)"
+        ". This is optional, and may be used to perform any plugin cleanup.",
+    )
+    _validate_deactivate_func = validator("on_deactivate", allow_reuse=True)(
+        _validators.python_name
     )
 
-    contributions: Optional[ContributionPoints]
+    contributions: Optional[ContributionPoints] = Field(
+        None,
+        description="An object describing the plugin's "
+        "[contributions](./contributions)",
+    )
 
-    _manifest_file: Optional[Path] = None
-    package_metadata: Optional[PackageMetadata] = None
+    package_metadata: Optional[PackageMetadata] = Field(
+        None,
+        description="Package metadata following "
+        "https://packaging.python.org/specifications/core-metadata/. "
+        "For normal (non-dynamic) plugins, this data will come from the package's "
+        "setup.cfg",
+        hide_docs=True,
+    )
 
     @property
     def license(self) -> Optional[str]:
         return self.package_metadata.license if self.package_metadata else None
 
     @property
-    def version(self) -> Optional[str]:
+    def package_version(self) -> Optional[str]:
         return self.package_metadata.version if self.package_metadata else None
 
     @property
@@ -135,43 +145,31 @@ class PluginManifest(BaseModel):
         return self.package_metadata.author if self.package_metadata else None
 
     @root_validator
-    def _check_engine_version(cls, values: dict) -> dict:
-        declared_version = Version.parse(values.get("engine", ""))
-        current_version = Version.parse(ENGINE_VERSION)
+    def _validate_root(cls, values: dict) -> dict:
+        # validate schema version
+        declared_version = Version.parse(values.get("schema_version", ""))
+        current_version = Version.parse(SCHEMA_VERSION)
         if current_version < declared_version:
             raise ValueError(
                 dedent(
-                    f"The declared engine version {declared_version} is "
-                    "newer than npe2 engine {current_version}. You may need to "
-                    "upgrade npe2."
+                    f"The declared schema version '{declared_version}' is "
+                    f"newer than npe2's schema version: '{current_version}'. You may "
+                    "need to upgrade npe2."
                 )
             )
-        return values
 
-    @validator("display_name")
-    def validate_display_name(cls, v):
-        if not _display_name_pattern.match(v):
-            raise ValueError(
-                f"{v} is not a valid display_name.  The display_name must "
-                "be 3-40 characters long, containing printable word characters, "
-                "and must not begin or end with an underscore, white space, or "
-                "non-word character."
-            )
-        return v
-
-    @root_validator
-    def _validate_root(cls, values: dict) -> dict:
+        mf_name = values.get("name")
         invalid_commands = []
         if values.get("contributions") is not None:
             for command in values["contributions"].commands or []:
                 id_start_actual = command.id.split(".")[0]
-                if values["name"] != id_start_actual:
+                if mf_name != id_start_actual:
                     invalid_commands.append(command.id)
 
         if invalid_commands:
             raise ValueError(
                 dedent(
-                    f"""Commands identifiers must start with the current package name {values['name']!r}
+                    f"""Commands identifiers must start with the current package name {mf_name!r}
             the following commands where found to break this assumption:
                 {invalid_commands}
             """
@@ -180,17 +178,6 @@ class PluginManifest(BaseModel):
 
         return values
 
-    def toml(self, pyproject=False) -> str:
-        with engine_in_fields_set(self):
-            d = json.loads(self.json(exclude_unset=True))
-            if pyproject:
-                d = {"tool": {"napari": d}}
-            return toml.dumps(d)
-
-    def yaml(self) -> str:
-        with engine_in_fields_set(self):
-            return yaml.safe_dump(json.loads(self.json(exclude_unset=True)))
-
     @classmethod
     def from_distribution(cls, name: str) -> PluginManifest:
         """Return PluginManifest given a distribution (package) name.
@@ -216,9 +203,7 @@ class PluginManifest(BaseModel):
         ValidationError
             If the manifest is not valid
         """
-        from importlib.metadata import distribution
-
-        dist = distribution(name)  # may raise PackageNotFoundError
+        dist = metadata.distribution(name)  # may raise PackageNotFoundError
         for ep in dist.entry_points:
             if ep.group == ENTRY_POINT:
                 return PluginManifest._from_entrypoint(ep, dist)
@@ -226,65 +211,10 @@ class PluginManifest(BaseModel):
             "Distribution {name!r} exists but does not provide a napari manifest"
         )
 
-    @classmethod
-    def from_file(cls, path: Union[Path, str]) -> PluginManifest:
-        """Parse PluginManifest from a specific file.
-
-        Parameters
-        ----------
-        path : Path or str
-            Path to a manifest.  Must have extension {'.json', '.yaml', '.yml', '.toml'}
-
-        Returns
-        -------
-        PluginManifest
-            The parsed manifest.
-
-        Raises
-        ------
-        FileNotFoundError
-            If `path` does not exist.
-        ValueError
-            If the file extension is not in {'.json', '.yaml', '.yml', '.toml'}
-        """
-        path = Path(path).expanduser().absolute().resolve()
-        if not path.exists():
-            raise FileNotFoundError(f"File not found: {path}")
-
-        loader: Callable
-        if path.suffix.lower() == ".json":
-            loader = json.load
-        elif path.suffix.lower() == ".toml":
-            loader = toml.load
-        elif path.suffix.lower() in (".yaml", ".yml"):
-            loader = yaml.safe_load
-        else:
-            raise ValueError(f"unrecognized file extension: {path}")
-
-        with open(path) as f:
-            data = loader(f) or {}
-
-        if path.name == "pyproject.toml":
-            data = data["tool"]["napari"]
-
-        mf = cls(**data)
-        mf._manifest_file = path
-        return mf
-
     class Config:
-        use_enum_values = True  # only needed for SPDX
         underscore_attrs_are_private = True
         extra = Extra.forbid
 
-    def _call_func_in_plugin_entrypoint(self, funcname: str, args=()) -> None:
-        """convenience to call a function in the plugins entry_point, if declared."""
-        if not self.entry_point:
-            return None
-        mod = import_module(self.entry_point)
-        func = getattr(mod, funcname, None)
-        if callable(func):
-            return func(*args)
-
     @classmethod
     def discover(
         cls,
@@ -325,8 +255,8 @@ class PluginManifest(BaseModel):
         DiscoverResults: (3 namedtuples: manifest, entrypoint, error)
             3-tuples with either manifest or (entrypoint and error) being None.
         """
-        with temporary_path_additions(paths):
-            for dist in distributions():
+        with _temporary_path_additions(paths):
+            for dist in metadata.distributions():
                 for ep in dist.entry_points:
                     if ep.group != entry_point_group:
                         continue
@@ -345,14 +275,16 @@ class PluginManifest(BaseModel):
 
     @classmethod
     def _from_entrypoint(
-        cls, entry_point: EntryPoint, distribution: Optional[Distribution] = None
+        cls,
+        entry_point: EntryPoint,
+        distribution: Optional[metadata.Distribution] = None,
     ) -> PluginManifest:
 
         match = entry_point.pattern.match(entry_point.value)  # type: ignore
         module = match.group("module")
 
         spec = util.find_spec(module or "")
-        if not spec:
+        if not spec:  # pragma: no cover
             raise ValueError(
                 f"Cannot find module {module!r} declared in "
                 f"entrypoint: {entry_point.value!r}"
@@ -372,7 +304,9 @@ class PluginManifest(BaseModel):
                     assert mf.name == meta.name, "Manifest name must match package name"
                     return mf
 
-        raise FileNotFoundError(f"Could not find file {fname!r} in module {module!r}")
+        raise FileNotFoundError(  # pragma: no cover
+            f"Could not find file {fname!r} in module {module!r}"
+        )
 
     @classmethod
     def _from_package_or_name(
@@ -406,12 +340,12 @@ class PluginManifest(BaseModel):
 
         try:
             return PluginManifest.from_file(package_or_filename)
-        except ValidationError:
+        except ValidationError:  # pragma: no cover
             raise
         except (FileNotFoundError, ValueError):
             try:
                 return PluginManifest.from_distribution(str(package_or_filename))
-            except ValidationError:
+            except ValidationError:  # pragma: no cover
                 raise
             except Exception as e:
                 raise ValueError(
@@ -419,11 +353,36 @@ class PluginManifest(BaseModel):
                     "package name or a file.."
                 ) from e
 
+    def validate_imports(self) -> None:
+        """Checks recursively that all `python_name` fields are actually importable."""
+        from .utils import import_python_name
+
+        errors = []
+
+        def check_pynames(m: BaseModel, loc=()):
+            for name, value in m:
+                if not value:
+                    continue
+                if isinstance(value, BaseModel):
+                    return check_pynames(value, (*loc, name))
+                field = m.__fields__[name]
+                if isinstance(value, list) and isinstance(field.type_, ModelMetaclass):
+                    return [check_pynames(i, (*loc, n)) for n, i in enumerate(value)]
+                if field.outer_type_ is PythonName:
+                    try:
+                        import_python_name(value)
+                    except (ImportError, AttributeError) as e:
+                        errors.append(ErrorWrapper(e, (*loc, name)))
+
+        check_pynames(self)
+        if errors:
+            raise ValidationError(errors, type(self))
+
     ValidationError = ValidationError  # for convenience of access
 
 
 @contextmanager
-def temporary_path_additions(paths: Sequence[Union[str, Path]] = ()):
+def _temporary_path_additions(paths: Sequence[Union[str, Path]] = ()):
     for p in reversed(paths):
         sys.path.insert(0, str(p))
     try:
@@ -433,16 +392,5 @@ def temporary_path_additions(paths: Sequence[Union[str, Path]] = ()):
             sys.path.remove(str(p))
 
 
-@contextmanager
-def engine_in_fields_set(manifest: PluginManifest):
-    was_there = "engine" in manifest.__fields_set__
-    manifest.__fields_set__.add("engine")
-    try:
-        yield
-    finally:
-        if not was_there:
-            manifest.__fields_set__.discard("engine")
-
-
 if __name__ == "__main__":
     print(PluginManifest.schema_json())
