@@ -16,6 +16,8 @@ from typing import (
     Union,
 )
 
+from ..types import PythonName
+
 if TYPE_CHECKING:
     from typing_extensions import Protocol
 
@@ -33,6 +35,8 @@ R = TypeVar("R")
 
 # TODO: add ParamSpec when it's supported better by mypy
 class Executable(Generic[R]):
+    command: str
+
     def exec(
         self: ProvidesCommand,
         args: tuple = (),
@@ -139,3 +143,26 @@ class Version:
 
     def __iter__(self):
         yield from self.to_tuple()
+
+    def __str__(self) -> str:
+        v = f"{self.major}.{self.minor}.{self.patch}"
+        if self.prerelease:  # pragma: no cover
+            v += str(self.prerelease)
+        if self.build:  # pragma: no cover
+            v += str(self.build)
+        return v
+
+
+def import_python_name(python_name: PythonName) -> Any:
+    from importlib import import_module
+
+    from ._validators import PYTHON_NAME_PATTERN
+
+    match = PYTHON_NAME_PATTERN.match(python_name)
+    if not match:  # pragma: no cover
+        raise ValueError(f"Invalid python name: {python_name}")
+
+    module_name, funcname = match.groups()
+
+    mod = import_module(module_name)
+    return getattr(mod, funcname)
